DEFAULTS {
    STACK_SIZE  = 4K
    IRQ_STACK_SIZE = 1K
    PE_RESERVED = 0x100
    GFX_RESERVED = 256K
}

MEMORY {
    gram         : org = 0x3F000000 + PE_RESERVED, len = 1M - PE_RESERVED - GFX_RESERVED
    gram_gfx     : org = 0x3F000000 + 1M - GFX_RESERVED, len = GFX_RESERVED
    sram         : org = 0x3EF00000, len = 1M
    iop_ram      : org = 0x3E400000, len = 32K
    cm4_tcm_code : org = 0x1E000000, len = 32K
    cm4_tcm_data : org = 0x3E000000, len = 32K
}

SECTIONS
{
    .text : {
        init.o(.text)
        *(.text)
        *(.rodata)
        . = ALIGN(0x100);
        vt.o(.text)
    } > sram

    .data : {
        _data_vma_s = .;
        *(.data)
        _data_vme_e = .;
     } > .
     _data_lma = _data_vma_s;

     .bss : {
         *(.bss)
     } > .

     .gfx : {} > gram_gfx

    .stack ALIGN(8) : {
        _stack_s = .;
        . = STACK_SIZE;
        _stack_e = .;
    } > .

    .irq_stack ALIGN(8) : {
        _irq_stack_s = .;
        . = IRQ_STACK_SIZE;
        _irq_stack_e = .;
    }

    .sdabase : {} > . /* required by GHS, but should be empty */

/*
 * These special symbols mark the bounds of RAM and ROM memory.
 * They are used by the MULTI debugger.
 */
    __ghs_ramstart  = MEMADDR(sram);
    __ghs_ramend    = MEMENDADDR(sram);
}
